﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ZylSoft.Serial;

namespace WpfDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private SerialPort serialPort = new SerialPort();        

        public MainWindow()
        {
            InitializeComponent();
            serialPort.Connected += SerialPortOnConnected;
            serialPort.Disconnected += SerialPortOnDisconnected;
            serialPort.Faulted += SerialPortOnFaulted;
            serialPort.Received += SerialPortOnReceived;
            serialPort.Interval = 200;
        }

        private void SerialPortOnDisconnected(object sender, ConnectionEventArgs connectionEventArgs)
        {            
            textBoxOut.Dispatcher.Invoke(new Action(delegate() { textBoxOut.Text += "Disconnected from: " + connectionEventArgs.Port + Environment.NewLine; textBoxOut.ScrollToEnd(); }), null);            
        }

        private void SerialPortOnConnected(object sender, ConnectionEventArgs connectionEventArgs)
        {
            textBoxOut.Dispatcher.Invoke(new Action(delegate() { textBoxOut.Text += "Connected to: " + connectionEventArgs.Port + Environment.NewLine; textBoxOut.ScrollToEnd(); }), null);           
        }

        private void SerialPortOnFaulted(object sender, ConnectionEventArgs connectionEventArgs)
        {
            textBoxOut.Dispatcher.Invoke(new Action(delegate() { textBoxOut.Text += "Faulted: " + connectionEventArgs.Port + Environment.NewLine; textBoxOut.ScrollToEnd(); }), null);
        }

        private void SerialPortOnReceived(object sender, DataEventArgs dataEventArgs)
        {
            textBoxOut.Dispatcher.Invoke(new Action(delegate() { textBoxOut.Text += SerialPort.ReplaceControlCharacters(SerialPort.AsciiByteArrayToString(dataEventArgs.Buffer)) + Environment.NewLine; textBoxOut.ScrollToEnd(); }), null);            
        }

        private void buttonOpen_Click(object sender, RoutedEventArgs e)
        {
            if (listBoxPorts.SelectedItems.Count < 1 || listBoxBaudRates.SelectedItems.Count < 1)
            {
                MessageBox.Show("Select port and baud rate!");
                return;
            }

            string port = listBoxPorts.SelectedItems[0].ToString();
            serialPort.Port = SerialPort.StringToSerialCommPort(port);            
            
            if (serialPort.Port == SerialCommPort.Custom)
            {
                serialPort.CustomPortName = port;
            }  

            ListBoxItem item = (ListBoxItem)listBoxBaudRates.SelectedItems[0];
            int baudRate = Convert.ToInt32(item.Content.ToString());
            serialPort.BaudRate = (SerialBaudRate)baudRate;
            if (!serialPort.Open())
                MessageBox.Show("Cannot connect to " + serialPort.Port.ToString());
        }

        private void buttonClose_Click(object sender, RoutedEventArgs e)
        {
            serialPort.Close();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            listBoxPorts.Items.Clear();
            String[] ports = ZylSoft.Serial.SerialPort.GetExistingCommPortNames();
            foreach (String i in ports)
            {
                listBoxPorts.Items.Add(i);
            }
        }    
    }
}
